@@ -35,6 +35,7 @@
 
 #include <string.h>
 #include <gst/gst.h>
+#include <gst/base/gstadapter.h>
 #include <gdk/gdk.h>
 
 #ifdef HAVE_GST_PBUTILS
@@ -46,11 +47,13 @@
 #  include <gst/interfaces/xoverlay.h>
 #endif
 
+#include "banshee-gst.h"
+
 #define P_INVOKE
 #define IS_BANSHEE_PLAYER(e) (e != NULL)
 #define SET_CALLBACK(cb_name) { if(player != NULL) { player->cb_name = cb; } }
 
-#define bp_debug g_debug
+#define bp_debug(x...) banshee_log_debug ("player", x)
 
 typedef struct BansheePlayer BansheePlayer;
 
@@ -62,9 +65,9 @@ typedef void (* BansheePlayerStateChangedCallback) (BansheePlayer *player, GstSt
 typedef void (* BansheePlayerIterateCallback)      (BansheePlayer *player);
 typedef void (* BansheePlayerBufferingCallback)    (BansheePlayer *player, gint buffering_progress);
 typedef void (* BansheePlayerTagFoundCallback)     (BansheePlayer *player, const gchar *tag, const GValue *value);
+typedef void (* BansheePlayerVisDataCallback)      (BansheePlayer *player, gint channels, gint samples, gfloat *data, gfloat *spectrum);
 
 struct BansheePlayer {
-
     // Player Callbacks
     BansheePlayerEosCallback eos_cb;
     BansheePlayerErrorCallback error_cb;
@@ -72,6 +75,7 @@ struct BansheePlayer {
     BansheePlayerIterateCallback iterate_cb;
     BansheePlayerBufferingCallback buffering_cb;
     BansheePlayerTagFoundCallback tag_found_cb;
+    BansheePlayerVisDataCallback vis_data_cb;
 
     // Pipeline Elements
     GstElement *playbin;
@@ -79,6 +83,8 @@ struct BansheePlayer {
     GstElement *audiobin;
     GstElement *equalizer;
     GstElement *preamp;
+    gint equalizer_status;
+    gdouble current_volume;
     
     // Pipeline/Playback State
     GMutex *mutex;
@@ -93,13 +99,36 @@ struct BansheePlayer {
     GdkWindow *video_window;
     #endif
     
+    // Visualization State
+    GstAdapter *vis_buffer;
+    gfloat *spectrum_buffer;
+    
     // Plugin Installer State
     GdkWindow *window;
     GSList *missing_element_details;
-    gboolean install_plugins_noprompt;
+    GSList *missing_element_details_handled;
+    gboolean handle_missing_elements;
     #ifdef HAVE_GST_PBUTILS
     GstInstallPluginsContext *install_plugins_context;
     #endif
+    
+    // ReplayGain State
+    gboolean replaygain_enabled;
+    
+    // ReplayGain history: stores the previous 10 scale factors
+    // and the current scale factor with the current at index 0
+    // and the oldest at index 10. History is used to compute 
+    // gain on a track where no adjustment information is present.
+    // http://replaygain.hydrogenaudio.org/player_scale.html
+    gdouble volume_scale_history[11];
+    gboolean volume_scale_history_shift;
+    gboolean current_scale_from_history;
+    
+    // ReplayGain cache
+    gdouble album_gain;
+    gdouble album_peak;
+    gdouble track_gain;
+    gdouble track_peak;
 };
 
 #endif /* _BANSHEE_PLAYER_PRIVATE_H */
