@@ -2,13 +2,13 @@ using System;
 using System.Reflection;
 using System.Runtime.CompilerServices;
 
-[assembly: ApplicationVersion ("1.5.5", "1.6 RC1")]
-[assembly: ApplicationBuildInformation ("git-checkout", "linux-gnu", "x86_64", "2010-03-10 17:09:51 PST")]
+[assembly: ApplicationVersion ("1.4.3", "1.4.3")]
+[assembly: ApplicationBuildInformation ("svn-checkout", "linux-gnu", "x86_64", "2009-03-04 17:42:52 EST")]
 
-[assembly: AssemblyVersion ("1.5.0.0")]
+[assembly: AssemblyVersion ("1.4.0.0")]
 [assembly: AssemblyTitle ("Banshee")]
 [assembly: AssemblyDescription ("Banshee Media Player")]
-[assembly: AssemblyCopyright ("Copyright (C) 2005-2010 Novell Inc. and others")]
+[assembly: AssemblyCopyright ("Copyright (C) 2005-2009 Novell Inc. and others")]
 [assembly: AssemblyCompany ("Novell, Inc.")]
 
 [AttributeUsage (AttributeTargets.Assembly, Inherited = false)]
