@@ -33,14 +33,19 @@ using Banshee.Base;
 
 namespace Banshee.Collection
 {
-    public class AlbumInfo 
+    public class AlbumInfo : CacheableItem
     {
         private string title;
         private string artist_name;
+        private bool is_compilation;
         private string artwork_id;
         private DateTime release_date = DateTime.MinValue;
         private string musicbrainz_id;
         
+        public AlbumInfo ()
+        {
+        }
+        
         public AlbumInfo (string title)
         {
             this.title = title;
@@ -56,6 +61,11 @@ namespace Banshee.Collection
             set { title = value; }
         }
         
+        public virtual bool IsCompilation {
+            get { return is_compilation; }
+            set { is_compilation = value; }
+        }
+        
         public virtual string MusicBrainzId {
             get { return musicbrainz_id; }
             set { musicbrainz_id = value; }
