@@ -53,6 +53,9 @@ namespace Banshee.MediaEngine
         private PlayerEngine active_engine;
         private PlayerEngine default_engine;
         private PlayerEngine pending_engine;
+        private object pending_playback_for_not_ready;
+        private bool pending_playback_for_not_ready_play;
+        private TrackInfo synthesized_contacting_track;
 
         private string preferred_engine_id = null;
 
@@ -183,6 +186,12 @@ namespace Banshee.MediaEngine
                         }
                     }
                 }
+
+                if (pending_playback_for_not_ready != null) {
+                    OpenCheck (pending_playback_for_not_ready, pending_playback_for_not_ready_play);
+                    pending_playback_for_not_ready = null;
+                    pending_playback_for_not_ready_play = false;
+                }
             }
             
             DBusPlayerStateHandler dbus_handler = dbus_state_changed;
@@ -196,10 +205,10 @@ namespace Banshee.MediaEngine
             if (CurrentTrack != null) {
                 if (args.Event == PlayerEvent.Error 
                     && CurrentTrack.PlaybackError == StreamPlaybackError.None) {
-                    CurrentTrack.PlaybackError = StreamPlaybackError.Unknown;
+                    CurrentTrack.SavePlaybackError (StreamPlaybackError.Unknown);
                 } else if (args.Event == PlayerEvent.Iterate 
                     && CurrentTrack.PlaybackError != StreamPlaybackError.None) {
-                    CurrentTrack.PlaybackError = StreamPlaybackError.None;
+                    CurrentTrack.SavePlaybackError (StreamPlaybackError.None);
                 }
             }
             
@@ -267,13 +276,12 @@ namespace Banshee.MediaEngine
         
         private void OpenPlay (TrackInfo track, bool play)
         {
-            if (!track.CanPlay || OnTrackIntercept (track)) {
+            if (track == null || !track.CanPlay || OnTrackIntercept (track)) {
                 return;
             }
         
             try {
-                OpenCheck (track);
-                active_engine.Play ();
+                OpenCheck (track, true);
             } catch (Exception e) {
                 Log.Exception (e);
                 Log.Error (Catalog.GetString ("Problem with Player Engine"), e.Message, true);
@@ -281,12 +289,18 @@ namespace Banshee.MediaEngine
                 ActiveEngine = default_engine;
             }
         }
-        
+
         private void OpenCheck (object o)
+        {
+            OpenCheck (o, false);
+        }
+        
+        private void OpenCheck (object o, bool play)
         {
             if (CurrentState == PlayerState.NotReady) {
-                throw new InvalidOperationException (String.Format ("Player engine {0} is in the NotReady state", 
-                    active_engine.GetType ().FullName));
+                pending_playback_for_not_ready = o;
+                pending_playback_for_not_ready_play = play;
+                return;
             }
         
             SafeUri uri = null;
@@ -313,18 +327,22 @@ namespace Banshee.MediaEngine
                 active_engine.Open (uri);
                 incremented_last_played = false;
             }
+
+            if (play) {
+                active_engine.Play ();
+            }
         }
 
         private bool incremented_last_played = true;
         public void IncrementLastPlayed ()
         {
-            if (!incremented_last_played && active_engine.CurrentTrack != null) {
+            if (!incremented_last_played && CurrentTrack != null && CurrentTrack.PlaybackError == StreamPlaybackError.None) {
                 // If we're at least 50% done playing a song, mark it as played, otherwise as skipped
                 // If the Length <= 0, assume the song was finished and increment its play count
                 if (active_engine.Length <= 0 || active_engine.Position >= active_engine.Length / 2) {
-                    active_engine.CurrentTrack.IncrementPlayCount ();
+                    CurrentTrack.IncrementPlayCount ();
                 } else {
-                    active_engine.CurrentTrack.IncrementSkipCount ();
+                    CurrentTrack.IncrementSkipCount ();
                 }
                 incremented_last_played = true;
             }
@@ -389,12 +407,32 @@ namespace Banshee.MediaEngine
                 active_engine.Pause ();
             }
         }
+
+        // For use by RadioTrackInfo
+        // TODO remove this method once RadioTrackInfo playlist downloading/parsing logic moved here?
+        internal void StartSynthesizeContacting (TrackInfo track)
+        {
+            //OnStateChanged (PlayerState.Contacting);
+            RaiseEvent (new PlayerEventStateChangeArgs (CurrentState, PlayerState.Contacting));
+            synthesized_contacting_track = track;
+        }
+
+        internal void EndSynthesizeContacting (TrackInfo track, bool idle)
+        {
+            if (track == synthesized_contacting_track) {
+                synthesized_contacting_track = null;
+
+                if (idle) {
+                    RaiseEvent (new PlayerEventStateChangeArgs (PlayerState.Contacting, PlayerState.Idle));
+                }
+            }
+        }
         
         public void TogglePlaying ()
         {
-            if (CurrentState == PlayerState.Playing) {
+            if (IsPlaying () && CurrentState != PlayerState.Paused) {
                 Pause ();
-            } else {
+            } else if (CurrentState != PlayerState.NotReady) {
                 Play ();
             }
         }
@@ -422,7 +460,9 @@ namespace Banshee.MediaEngine
         {
             return CurrentState == PlayerState.Playing || 
                 CurrentState == PlayerState.Paused || 
-                CurrentState == PlayerState.Loaded;
+                CurrentState == PlayerState.Loaded ||
+                CurrentState == PlayerState.Loading ||
+                CurrentState == PlayerState.Contacting;
         }
 
         private void CheckPending ()
@@ -438,7 +478,7 @@ namespace Banshee.MediaEngine
         }
     
         public TrackInfo CurrentTrack {
-            get { return active_engine.CurrentTrack; }
+            get { return active_engine.CurrentTrack ?? synthesized_contacting_track; }
         }
         
         private Dictionary<string, object> dbus_sucks;
@@ -464,7 +504,7 @@ namespace Banshee.MediaEngine
         }
         
         public PlayerState CurrentState {
-            get { return active_engine.CurrentState; }
+            get { return synthesized_contacting_track != null ? PlayerState.Contacting : active_engine.CurrentState; }
         }
         
         string IPlayerEngineService.CurrentState {
@@ -514,11 +554,11 @@ namespace Banshee.MediaEngine
                 uint length = active_engine.Length;
                 if (length > 0) {
                     return length;
-                } else if (active_engine.CurrentTrack == null) {
+                } else if (CurrentTrack == null) {
                     return 0;
                 }
                 
-                return (uint) active_engine.CurrentTrack.Duration.TotalSeconds;
+                return (uint) CurrentTrack.Duration.TotalSeconds;
             }
         }
     
