@@ -37,7 +37,7 @@ using Mono.Unix;
 using Hyena;
 using Hyena.Query;
 using Hyena.Data.Sqlite;
-
+ 
 using Banshee.Base;
 using Banshee.Query;
 using Banshee.Sources;
@@ -55,7 +55,7 @@ namespace Banshee.SmartPlaylist
     {
         private static List<SmartPlaylistSource> playlists = new List<SmartPlaylistSource> ();
         private static uint timeout_id = 0;
-
+        
         static SmartPlaylistSource () {
             Migrator.MigrateAll ();
 
@@ -65,13 +65,13 @@ namespace Banshee.SmartPlaylist
 
         private static string generic_name = Catalog.GetString ("Smart Playlist");
         private static string properties_label = Catalog.GetString ("Edit Smart Playlist");
-
+    
         private QueryOrder query_order;
         private QueryLimit limit;
         private IntegerQueryValue limit_value;
-
+        
         private List<SmartPlaylistSource> dependencies = new List<SmartPlaylistSource>();
-
+        
 
 #region Properties
 
@@ -116,14 +116,11 @@ namespace Banshee.SmartPlaylist
                 if (condition != null) {
                     condition_sql = condition.ToSql (BansheeQuery.FieldSet);
                     condition_xml = condition.ToXml (BansheeQuery.FieldSet);
-
+                    
                     foreach (QueryTermNode term in condition.GetTerms ()) {
                         if (!relevant_fields.Contains (term.Field))
                             relevant_fields.Add (term.Field);
                     }
-                } else {
-                    condition_sql = null;
-                    condition_xml = null;
                 }
             }
         }
@@ -179,18 +176,12 @@ namespace Banshee.SmartPlaylist
                 return (Limit != null && LimitValue != null && !LimitValue.IsEmpty && QueryOrder != null);
             }
         }
-
-        public bool IsHiddenWhenEmpty { get; private set; }
-
+        
         public override bool HasDependencies {
             get { return dependencies.Count > 0; }
         }
 
         // FIXME scan ConditionTree for date fields
-        // FIXME even better, scan for date fields and see how fine-grained they are;
-        //       eg if set to Last Added < 2 weeks ago, don't need a timer going off
-        //       every 1 minute - every hour or two would suffice.  Just change this
-        //       property to a TimeSpan, rename it TimeDependentResolution or something
         public bool TimeDependent {
             get { return false; }
         }
@@ -205,25 +196,19 @@ namespace Banshee.SmartPlaylist
         }
 
         public SmartPlaylistSource (string name, QueryNode condition, QueryOrder order, QueryLimit limit, IntegerQueryValue limit_value, PrimarySource parent)
-            : this (name, condition, order, limit, limit_value, false, parent)
-        {
-        }
-
-        public SmartPlaylistSource (string name, QueryNode condition, QueryOrder order, QueryLimit limit, IntegerQueryValue limit_value, bool hiddenWhenEmpty, PrimarySource parent)
             : this (name, parent)
         {
             ConditionTree = condition;
             QueryOrder = order;
             Limit = limit;
             LimitValue = limit_value;
-            IsHiddenWhenEmpty = hiddenWhenEmpty;
 
             SetProperties ();
             UpdateDependencies ();
         }
 
         // For existing smart playlists that we're loading from the database
-        protected SmartPlaylistSource (int dbid, string name, string condition_xml, string order_by, string limit_number, string limit_criterion, PrimarySource parent, int count, bool is_temp, bool hiddenWhenEmpty) :
+        protected SmartPlaylistSource (int dbid, string name, string condition_xml, string order_by, string limit_number, string limit_criterion, PrimarySource parent, int count, bool is_temp) :
             base (generic_name, name, dbid, -1, 0, parent, is_temp)
         {
             ConditionXml = condition_xml;
@@ -232,7 +217,6 @@ namespace Banshee.SmartPlaylist
             LimitValue = new IntegerQueryValue ();
             LimitValue.ParseUserQuery (limit_number);
             SavedCount = count;
-            IsHiddenWhenEmpty = hiddenWhenEmpty;
 
             SetProperties ();
             UpdateDependencies ();
@@ -260,17 +244,17 @@ namespace Banshee.SmartPlaylist
         }
 
 #endregion
-
+        
 #region Private Methods
-
+        
         private void UpdateDependencies ()
         {
             foreach (SmartPlaylistSource s in dependencies) {
                 s.Updated -= OnDependencyUpdated;
             }
-
+            
             dependencies.Clear ();
-
+            
             if (ConditionTree != null) {
                 foreach (SmartPlaylistQueryValue value in ConditionTree.SearchForValues<SmartPlaylistQueryValue> ()) {
                     SmartPlaylistSource playlist = value.ObjectValue;
@@ -281,12 +265,12 @@ namespace Banshee.SmartPlaylist
                 }
             }
         }
-
+        
         private void OnDependencyUpdated (object sender, EventArgs args)
         {
             Reload ();
         }
-
+        
 #endregion
 
 #region AbstractPlaylist overrides
@@ -300,23 +284,19 @@ namespace Banshee.SmartPlaylist
                 PrimarySource.TracksChanged += HandleTracksChanged;
                 PrimarySource.TracksDeleted += HandleTracksDeleted;
             }
-
-            if (IsHiddenWhenEmpty) {
-                RefreshAndReload ();
-            }
         }
 
         protected override void Create ()
         {
             DbId = ServiceManager.DbConnection.Execute (new HyenaSqliteCommand (@"
                 INSERT INTO CoreSmartPlaylists
-                    (Name, Condition, OrderBy, LimitNumber, LimitCriterion, PrimarySourceID, IsTemporary, IsHiddenWhenEmpty)
-                    VALUES (?, ?, ?, ?, ?, ?, ?, ?)",
+                    (Name, Condition, OrderBy, LimitNumber, LimitCriterion, PrimarySourceID, IsTemporary)
+                    VALUES (?, ?, ?, ?, ?, ?, ?)",
                 Name, ConditionXml,
                 IsLimited ? QueryOrder.Name : null,
                 IsLimited ? LimitValue.ToSql () : null,
                 IsLimited ? Limit.Name : null,
-                PrimarySourceId, IsTemporary, IsHiddenWhenEmpty
+                PrimarySourceId, IsTemporary
             ));
             UpdateDependencies ();
         }
@@ -331,14 +311,13 @@ namespace Banshee.SmartPlaylist
                         LimitNumber = ?,
                         LimitCriterion = ?,
                         CachedCount = ?,
-                        IsTemporary = ?,
-                        IsHiddenWhenEmpty = ?
+                        IsTemporary = ?
                     WHERE SmartPlaylistID = ?",
                 Name, ConditionXml,
                 IsLimited ? QueryOrder.Name : null,
                 IsLimited ? LimitValue.ToSql () : null,
                 IsLimited ? Limit.Name : null,
-                Count, IsTemporary, IsHiddenWhenEmpty, DbId
+                Count, IsTemporary, DbId
             ));
             UpdateDependencies ();
         }
@@ -370,34 +349,18 @@ namespace Banshee.SmartPlaylist
         private bool refreshed = false;
         public override void Reload ()
         {
-            if (!refreshed) {
+            if (!refreshed)
                 Refresh ();
-            } else {
-                // Don't set this on the first refresh
-                Properties.Set<bool> ("NotifyWhenAdded", IsHiddenWhenEmpty);
-            }
 
             base.Reload ();
-
-            if (IsHiddenWhenEmpty && Parent != null) {
-                bool contains_me = Parent.ContainsChildSource (this);
-                int count = Count;
-
-                if (count == 0 && contains_me) {
-                    Parent.RemoveChildSource (this);
-                } else if (count > 0 && !contains_me) {
-                    Parent.AddChildSource (this);
-                }
-            }
         }
 
         public void Refresh ()
         {
-            // Wipe the member list clean and repopulate it
+            // Wipe the member list clean and repopulate it 
             string reload_str = String.Format (
                 @"DELETE FROM CoreSmartPlaylistEntries WHERE SmartPlaylistID = {0};
-                  INSERT INTO CoreSmartPlaylistEntries
-                    (EntryID, SmartPlaylistID, TrackID)
+                  INSERT INTO CoreSmartPlaylistEntries 
                     SELECT NULL, {0} as SmartPlaylistID, TrackId FROM {1}
                         WHERE {2} AND CoreTracks.PrimarySourceID = {3}
                         {4} {5} {6}",
@@ -410,7 +373,7 @@ namespace Banshee.SmartPlaylist
             if (IsLimited && !Limit.RowBased) {
                 // Identify where the cut off mark is
                 HyenaSqliteCommand limit_command = new HyenaSqliteCommand (String.Format (
-                    @"SELECT EntryID, {0}
+                    @"SELECT EntryID, {0} 
                       FROM CoreTracks, CoreSmartPlaylistEntries
                       WHERE SmartPlaylistID = {1} AND CoreSmartPlaylistEntries.TrackID = CoreTracks.TrackID
                       ORDER BY EntryID",
@@ -523,7 +486,7 @@ namespace Banshee.SmartPlaylist
         {
             ClearTemporary ();
             using (HyenaDataReader reader = new HyenaDataReader (ServiceManager.DbConnection.Query (
-                @"SELECT SmartPlaylistID, Name, Condition, OrderBy, LimitNumber, LimitCriterion, PrimarySourceID, CachedCount, IsTemporary, IsHiddenWhenEmpty
+                @"SELECT SmartPlaylistID, Name, Condition, OrderBy, LimitNumber, LimitCriterion, PrimarySourceID, CachedCount, IsTemporary
                     FROM CoreSmartPlaylists WHERE PrimarySourceID = ?", parent.DbId))) {
                 while (reader.Read ()) {
                     SmartPlaylistSource playlist = null;
@@ -532,13 +495,12 @@ namespace Banshee.SmartPlaylist
                             reader.Get<int> (0), reader.Get<string> (1),
                             reader.Get<string> (2), reader.Get<string> (3),
                             reader.Get<string> (4), reader.Get<string> (5),
-                            parent, reader.Get<int> (7), reader.Get<bool> (8),
-                            reader.Get<bool> (9)
+                            parent, reader.Get<int> (7), reader.Get<bool> (8)
                         );
                     } catch (Exception e) {
                         Log.Warning ("Ignoring Smart Playlist", String.Format ("Caught error: {0}", e), false);
                     }
-
+                    
                     if (playlist != null) {
                         yield return playlist;
                     }
@@ -546,14 +508,18 @@ namespace Banshee.SmartPlaylist
             }
         }
 
+        private static bool temps_cleared = false;
         private static void ClearTemporary ()
         {
-            ServiceManager.DbConnection.Execute (@"
-                BEGIN TRANSACTION;
-                    DELETE FROM CoreSmartPlaylistEntries WHERE SmartPlaylistID IN (SELECT SmartPlaylistID FROM CoreSmartPlaylists WHERE IsTemporary = 1);
-                    DELETE FROM CoreSmartPlaylists WHERE IsTemporary = 1;
-                COMMIT TRANSACTION"
-            );
+            if (!temps_cleared) {
+                temps_cleared = true;
+                ServiceManager.DbConnection.Execute (@"
+                    BEGIN TRANSACTION;
+                        DELETE FROM CoreSmartPlaylistEntries WHERE SmartPlaylistID IN (SELECT SmartPlaylistID FROM CoreSmartPlaylists WHERE IsTemporary = 1);
+                        DELETE FROM CoreSmartPlaylists WHERE IsTemporary = 1;
+                    COMMIT TRANSACTION"
+                );
+            }
         }
 
         private static void HandleSourceAdded (SourceEventArgs args)
