@@ -49,7 +49,7 @@ namespace Banshee.Sources
         public int Position;
     }
     
-    public class SourceManager : ISourceManager, IInitializeService, IRequiredService, IDisposable
+    public class SourceManager : /*ISourceManager,*/ IInitializeService, IRequiredService, IDBusExportable, IDisposable
     {
         private List<Source> sources = new List<Source>();
         private Dictionary<string, Source> extension_sources = new Dictionary<string, Source> ();
@@ -114,7 +114,13 @@ namespace Banshee.Sources
                 if (args.Change == ExtensionChange.Add && !extension_sources.ContainsKey (node.Id)) {
                     Source source = (Source)node.CreateInstance ();
                     extension_sources.Add (node.Id, source);
-                    AddSource (source);
+                    bool add_source = true;
+                    if (source.Properties.Contains ("AutoAddSource")) {
+                        add_source = source.Properties.GetBoolean ("AutoAddSource");
+                    }
+                    if (add_source) {
+                        AddSource (source);
+                    }
                 } else if (args.Change == ExtensionChange.Remove && extension_sources.ContainsKey (node.Id)) {
                     Source source = extension_sources[node.Id];
                     extension_sources.Remove (node.Id);
@@ -130,6 +136,7 @@ namespace Banshee.Sources
         
         public void AddSource(Source source, bool isDefault)
         {
+            Banshee.Base.ThreadAssist.AssertInMainThread ();
             if(source == null || ContainsSource (source)) {
                 return;
             }
@@ -158,11 +165,15 @@ namespace Banshee.Sources
             } else if (source is VideoLibrarySource) {
                 video_library = source as VideoLibrarySource;
             }
-
-            ServiceManager.DBusServiceManager.RegisterObject(source);
             
-            foreach(Source child_source in source.Children) {
-                AddSource(child_source, false);
+            IDBusExportable exportable = source as IDBusExportable;
+            if (exportable != null) {
+                ServiceManager.DBusServiceManager.RegisterObject (exportable);
+            }
+            
+            List<Source> children = new List<Source> (source.Children);
+            foreach(Source child_source in children) {
+                AddSource (child_source, false);
             }
                 
             if(isDefault && ActiveSource == null) {
@@ -194,6 +205,11 @@ namespace Banshee.Sources
             foreach(Source child_source in source.Children) {
                 RemoveSource (child_source, recursivelyDispose);
             }
+            
+            IDBusExportable exportable = source as IDBusExportable;
+            if (exportable != null) {
+                ServiceManager.DBusServiceManager.UnregisterObject (exportable);
+            }
 
             if (recursivelyDispose) {
                 IDisposable disposable = source as IDisposable;
@@ -202,16 +218,18 @@ namespace Banshee.Sources
                 }
             }
 
-            if(source == active_source) {
-                SetActiveSource(default_source);
-            }
-                
-            SourceEventHandler handler = SourceRemoved;
-            if(handler != null) {
-                SourceEventArgs args = new SourceEventArgs();
-                args.Source = source;
-                handler(args);
-            }
+            Banshee.Base.ThreadAssist.ProxyToMain (delegate {
+                if(source == active_source) {
+                    SetActiveSource(default_source);
+                }
+
+                SourceEventHandler handler = SourceRemoved;
+                if(handler != null) {
+                    SourceEventArgs args = new SourceEventArgs();
+                    args.Source = source;
+                    handler(args);
+                }
+            });
         }
         
         public void RemoveSource(Type type)
@@ -290,14 +308,14 @@ namespace Banshee.Sources
             get { return active_source; }
         }
         
-        ISource ISourceManager.DefaultSource {
+        /*ISource ISourceManager.DefaultSource {
             get { return DefaultSource; }
         }
         
         ISource ISourceManager.ActiveSource {
             get { return ActiveSource; }
             set { value.Activate (); }
-        }
+        }*/
         
         public void SetActiveSource(Source source)
         {
@@ -306,7 +324,8 @@ namespace Banshee.Sources
         
         public void SetActiveSource(Source source, bool notify)
         {
-            if(source == null || active_source == source) {
+            Banshee.Base.ThreadAssist.AssertInMainThread ();
+            if(source == null || !source.CanActivate || active_source == source) {
                 return;
             }
             
@@ -330,14 +349,24 @@ namespace Banshee.Sources
             
             source.Activate();
         }
+
+        public IEnumerable<T> FindSources<T> () where T : Source
+        {
+            foreach (Source source in Sources) {
+                T t_source = source as T;
+                if (t_source != null) {
+                    yield return t_source;
+                }
+            }
+        }
      
         public ICollection<Source> Sources {
             get { return sources; }
         }
         
-        string [] ISourceManager.Sources {
+        /*string [] ISourceManager.Sources {
             get { return DBusServiceManager.MakeObjectPathArray<Source>(sources); }
-        }
+        }*/
         
         IDBusExportable IDBusExportable.Parent {
             get { return null; }
