@@ -40,9 +40,7 @@ namespace Banshee.Gui.TrackEditor
     {
         public delegate string FieldLabelClosure (EditorTrackInfo track, Widget widget);
         public delegate void FieldValueClosure (EditorTrackInfo track, Widget widget);
-        
-        public event EventHandler Changed;
-        
+
         private TrackEditorDialog dialog;
         protected TrackEditorDialog Dialog {
             get { return dialog; }
@@ -53,7 +51,7 @@ namespace Banshee.Gui.TrackEditor
             get { return current_track; }
         }
         
-        private struct FieldSlot
+        public struct FieldSlot
         {
             public Widget Label;
             public Widget Field;
@@ -82,20 +80,6 @@ namespace Banshee.Gui.TrackEditor
         {
         }
         
-        protected virtual void OnChanged ()
-        {
-            if (current_track == null) {
-                return;
-            }
-            
-            current_track.Changed = true;
-            
-            EventHandler handler = Changed;
-            if (handler != null) {
-                handler (this, EventArgs.Empty);
-            }
-        }
-        
         public virtual bool MultipleTracks {
             get { return dialog.TrackCount > 1; }
         }
@@ -112,27 +96,27 @@ namespace Banshee.Gui.TrackEditor
             get { return PageType.Edit; }
         }
     
-        public void AddField (Box parent, Widget field, string syncTooltip, FieldLabelClosure labelClosure, 
+        public FieldSlot AddField (Box parent, Widget field, string syncTooltip, FieldLabelClosure labelClosure, 
             FieldValueClosure readClosure, FieldValueClosure writeClosure)
         {
-            AddField (parent, EditorUtilities.CreateLabel (String.Empty), field, syncTooltip,
+            return AddField (parent, EditorUtilities.CreateLabel (String.Empty), field, syncTooltip,
                 labelClosure, readClosure, writeClosure, FieldOptions.None);
         }
         
-        public void AddField (Box parent, Widget field, string syncTooltip, FieldLabelClosure labelClosure, 
+        public FieldSlot AddField (Box parent, Widget field, string syncTooltip, FieldLabelClosure labelClosure, 
             FieldValueClosure readClosure, FieldValueClosure writeClosure, FieldOptions options)
         {
-            AddField (parent, EditorUtilities.CreateLabel (String.Empty), field, syncTooltip,
+            return AddField (parent, EditorUtilities.CreateLabel (String.Empty), field, syncTooltip,
                 labelClosure, readClosure, writeClosure, options);
         }
         
-        public void AddField (Box parent, Widget label, Widget field, string syncTooltip, 
+        public FieldSlot AddField (Box parent, Widget label, Widget field, string syncTooltip, 
             FieldLabelClosure labelClosure, FieldValueClosure readClosure, FieldValueClosure writeClosure)
         {
-            AddField (parent, label, field, syncTooltip, labelClosure, readClosure, writeClosure, FieldOptions.None);
+            return AddField (parent, label, field, syncTooltip, labelClosure, readClosure, writeClosure, FieldOptions.None);
         }
         
-        public void AddField (Box parent, Widget label, Widget field, string syncTooltip, FieldLabelClosure labelClosure, 
+        public FieldSlot AddField (Box parent, Widget label, Widget field, string syncTooltip, FieldLabelClosure labelClosure, 
             FieldValueClosure readClosure, FieldValueClosure writeClosure, FieldOptions options)
         {
             FieldSlot slot = new FieldSlot ();
@@ -166,7 +150,6 @@ namespace Banshee.Gui.TrackEditor
             IEditorField editor_field = field as IEditorField;
             if (editor_field != null) {
                 editor_field.Changed += delegate {
-                    OnChanged ();
                     if (CurrentTrack != null) {
                         slot.WriteClosure (CurrentTrack, slot.Field);
                     }
@@ -178,10 +161,12 @@ namespace Banshee.Gui.TrackEditor
                     AttachOptions.Fill, 
                     AttachOptions.Fill, 0, 0);
             }
-            
-            table.Attach (label, 0, table.NColumns, 0, 1,
-                AttachOptions.Fill | AttachOptions.Expand, 
-                AttachOptions.Fill, 0, 0);
+
+            if (label != null) {
+                table.Attach (label, 0, table.NColumns, 0, 1,
+                    AttachOptions.Fill | AttachOptions.Expand, 
+                    AttachOptions.Fill, 0, 0);
+            }
                 
             table.ShowAll ();
             
@@ -193,6 +178,8 @@ namespace Banshee.Gui.TrackEditor
                 shrink.PackStart (table, false, false, 0);
                 parent.PackStart (shrink, false, false, 0);
             }
+            
+            return slot;
         }
         
         public virtual void LoadTrack (EditorTrackInfo track)
@@ -209,10 +196,10 @@ namespace Banshee.Gui.TrackEditor
         
         private void UpdateLabel (EditorTrackInfo track, FieldSlot slot)
         {
-            string value = slot.LabelClosure (track, slot.Label);
             Label label = slot.Label as Label;
-            if (value != null && label != null) {
-                label.Text = value;
+            if (label != null && slot.LabelClosure != null) {
+                string value = slot.LabelClosure (track, slot.Label);
+                label.Text = value ?? String.Empty;
             }
         }
     }
