@@ -52,6 +52,7 @@ namespace Banshee.Sources.Gui
 {
     public abstract class FilteredListSourceContents : VBox, ISourceContents
     {
+        private string name;
         private object main_view;
         private Gtk.ScrolledWindow main_scrolled_window;
         
@@ -81,17 +82,22 @@ namespace Banshee.Sources.Gui
             </ui>
         ";
 
-        public FilteredListSourceContents ()
+        public FilteredListSourceContents (string name)
         {
+            this.name = name;
             InitializeViews ();
-        
-            string position = BrowserPosition.Get ();
+            
+            string position = ForcePosition == null ? BrowserPosition.Get () : ForcePosition;
             if (position == "top") {
                 LayoutTop ();
             } else {
                 LayoutLeft ();
             }
             
+            if (ForcePosition != null) {
+                return;
+            }
+            
             if (ServiceManager.Contains ("InterfaceActionService")) {
                 action_service = ServiceManager.Get<InterfaceActionService> ();
                 
@@ -126,7 +132,9 @@ namespace Banshee.Sources.Gui
             }
             
             ServiceManager.SourceManager.ActiveSourceChanged += delegate {
-                browser_container.Visible = ActiveSourceCanHasBrowser ? BrowserVisible.Get () : false; 
+                Banshee.Base.ThreadAssist.ProxyToMain (delegate {
+                    browser_container.Visible = ActiveSourceCanHasBrowser ? BrowserVisible.Get () : false;
+                });
             };
             
             NoShowAll = true;
@@ -141,7 +149,8 @@ namespace Banshee.Sources.Gui
         
         protected void SetupFilterView<T> (ListView<T> filter_view)
         {
-            filter_scrolled_windows.Add (SetupView (filter_view));
+            ScrolledWindow window = SetupView (filter_view);
+            filter_scrolled_windows.Add (window);
             filter_view.HeaderVisible = false;
             filter_view.SelectionProxy.Changed += OnBrowserViewSelectionChanged;
         }
@@ -165,88 +174,101 @@ namespace Banshee.Sources.Gui
         
         private void Reset ()
         {
-            //Hyena.Log.Information ("ListBrowser Reset");
             // Unparent the views' scrolled window parents so they can be re-packed in 
             // a new layout. The main container gets destroyed since it will be recreated.
             
-            if (filter_scrolled_windows.Count > 0) {
-                Container filter_container = filter_scrolled_windows[0].Parent as Container;
+            foreach (ScrolledWindow window in filter_scrolled_windows) {
+                Paned filter_container = window.Parent as Paned;
                 if (filter_container != null) {
-                    foreach (ScrolledWindow window in filter_scrolled_windows) {
-                        filter_container.Remove (window);
-                    }
-                } //else
-                    //Hyena.Log.Information ("No filter container");
-            } //else
-                //Hyena.Log.Information ("No filter windows");
-            
+                    filter_container.Remove (window);
+                }
+            }
+
             if (container != null && main_scrolled_window != null) {
                 container.Remove (main_scrolled_window);
-            } //else
-                //Hyena.Log.Information ("No main container");
+            }
             
             if (container != null) {
                 Remove (container);
-                container.Destroy ();
             }
         }
 
         private void LayoutLeft ()
         {
-            //Hyena.Log.Information ("ListBrowser LayoutLeft");
-            Reset ();
-            
-            container = new HPaned ();
-            VBox filter_box = new VBox ();
-            filter_box.Spacing = 10;
-            
-            foreach (ScrolledWindow window in filter_scrolled_windows) {
-                filter_box.PackStart (window, true, true, 0);
-            }
-            
-            //filter_box.Position = 350;
-            //PersistentPaneController.Control (filter_box, "browser.left.artist_album_box");
-            
-            container.Add1 (filter_box);
-            container.Add2 (main_scrolled_window);
-            
-            browser_container = filter_box;
-            
-            container.Position = 275;
-            PersistentPaneController.Control (container, "browser.left");
-            ShowPack ();
+            Layout (false);
         }
         
         private void LayoutTop ()
         {
-            //Hyena.Log.Information ("ListBrowser LayoutTop");
+            Layout (true);
+        }
+
+        private void Layout (bool top)
+        {
+            //Hyena.Log.Information ("ListBrowser LayoutLeft");
             Reset ();
             
-            container = new VPaned ();
-            HBox filter_box = new HBox ();
-            filter_box.Spacing = 10;
+            container = GetPane (!top);
+            Paned filter_box = GetPane (top);
+            filter_box.PositionSet = true;
+            Paned current_pane = filter_box;
             
-            foreach (ScrolledWindow window in filter_scrolled_windows) {
-                filter_box.PackStart (window, true, true, 0);
+            for (int i = 0; i < filter_scrolled_windows.Count; i++) {
+                ScrolledWindow window = filter_scrolled_windows[i];
+                bool last_even_filter = (i == filter_scrolled_windows.Count - 1 && filter_scrolled_windows.Count % 2 == 0);
+                if (i > 0 && !last_even_filter) {
+                    Paned new_pane = GetPane (top);
+                    current_pane.Pack2 (new_pane, true, false);
+                    current_pane.Position = 350;
+                    PersistentPaneController.Control (current_pane, ControllerName (top, i));
+                    current_pane = new_pane;
+                }
+               
+                if (last_even_filter) {
+                    current_pane.Pack2 (window, true, false);
+                    current_pane.Position = 350;
+                    PersistentPaneController.Control (current_pane, ControllerName (top, i));
+                } else {
+                    /*if (i == 0)
+                        current_pane.Pack1 (window, false, false);
+                    else*/
+                        current_pane.Pack1 (window, true, false);
+                }
+                    
             }
             
-            container.Add1 (filter_box);
-            container.Add2 (main_scrolled_window);
-            
+            container.Pack1 (filter_box, true, false);
+            container.Pack2 (main_scrolled_window, true, false);
             browser_container = filter_box;
             
-            container.Position = 175;
-            PersistentPaneController.Control (container, "browser.top");
+            container.Position = top ? 175 : 275;
+            PersistentPaneController.Control (container, ControllerName (top, -1));
             ShowPack ();
         }
         
+        private string ControllerName (bool top, int filter)
+        {
+            if (filter == -1)
+                return String.Format ("{0}.browser.{1}", name, top ? "top" : "left");
+            else
+                return String.Format ("{0}.browser.{1}.{2}", name, top ? "top" : "left", filter);
+        }
+        
+        private Paned GetPane (bool hpane)
+        {
+            if (hpane)
+                return new HPaned ();
+            else
+                return new VPaned ();
+        }
+        
         private void ShowPack ()
         {
             PackStart (container, true, true, 0);
             NoShowAll = false;
             ShowAll ();
             NoShowAll = true;
-            browser_container.Visible = BrowserVisible.Get ();
+            browser_container.Visible = ForcePosition != null || BrowserVisible.Get ();
         }
         
         private void OnViewModeChanged (object o, ChangedArgs args)
@@ -277,12 +299,14 @@ namespace Banshee.Sources.Gui
         
         protected virtual void OnBrowserViewSelectionChanged (object o, EventArgs args)
         {
+            // If the All item is now selected, scroll to the top
             Hyena.Collections.Selection selection = (Hyena.Collections.Selection) o;
-
             if (selection.AllSelected) {
+                // Find the view associated with this selection; a bit yuck; pass view in args?
                 foreach (IListView view in filter_views) {
                     if (view.Selection == selection) {
                         view.ScrollTo (0);
+                        break;
                     }
                 }
             }
@@ -303,9 +327,15 @@ namespace Banshee.Sources.Gui
             if (view.Model != null) {
                 model_positions[view.Model] = view.Vadjustment.Value;
             }
+
+            if (model == null) {
+                view.SetModel (null);
+                return;
+            }
             
-            if (!model_positions.ContainsKey (model))
-                    model_positions[model] = 0.0;
+            if (!model_positions.ContainsKey (model)) {
+                model_positions[model] = 0.0;
+            }
             
             view.SetModel (model, model_positions[model]);
         }
@@ -321,6 +351,10 @@ namespace Banshee.Sources.Gui
 
             return null;
         }
+        
+        protected virtual string ForcePosition {
+            get { return null; }
+        }
 
         protected abstract bool ActiveSourceCanHasBrowser { get; }
 
