@@ -44,6 +44,7 @@ namespace Hyena.Data.Gui
         
         public ColumnCellRating (string property, bool expand) : base (property, expand)
         {
+            Xpad = 0;
         }
             
         public override void Render (CellContext context, StateType state, double cellWidth, double cellHeight)
@@ -51,7 +52,7 @@ namespace Hyena.Data.Gui
             Gdk.Rectangle area = new Gdk.Rectangle (0, 0, context.Area.Width, context.Area.Height);
             
             // FIXME: Compute font height and set to renderer.Size
-            
+
             renderer.Value = Value;
             renderer.Render (context.Context, area, context.Theme.Colors.GetWidgetColor (GtkColorClass.Text, state), 
                 hover_bound == BoundObjectParent && hover_bound != null, hover_value, 0.8, 0.45, 0.35);
@@ -98,10 +99,9 @@ namespace Hyena.Data.Gui
             return true;
         }
         
-        public void GetSize (out int width, out int height)
+        public void GetWidthRange (Pango.Layout layout, out int min, out int max)
         {
-            width = renderer.Width;
-            height = renderer.Height;
+            min = max = renderer.Width;
         }
         
         private int RatingFromPosition (double x)
@@ -116,7 +116,7 @@ namespace Hyena.Data.Gui
         }
         
         private int Value {
-            get { return renderer.ClampValue ((int)BoundObject); }
+            get { return BoundObject == null ? MinRating : renderer.ClampValue ((int)BoundObject); }
             set { BoundObject = renderer.ClampValue (value); }
         }
         
